# File hectospec/Scripts/hgain1.cl
# By Doug Mink after Bill Wyatt
# February 29, 2008

# Run the 4 separate gain corrections for HECTOSPEC

procedure hgain1 (image)

string image = ""        {prompt="Image to correct"}

real	g1 = 1.0	{prompt="Gain correction for amplifier 1"}
real	g2 = 1.01	{prompt="Gain correction for amplifier 2"}
real	g3 = 1.032	{prompt="Gain correction for amplifier 3"}
real	g4 = 1.042	{prompt="Gain correction for amplifier 4"}
bool	saveold=yes     {prompt="Save unprocessed image files (y or n)"}
bool	deltemp=yes	{prompt="Save single-extension files (y or n)"}
bool	verbose=yes	{prompt="Verbose output (y or n)"}
bool	debug=no	{prompt="Extra debugging output (y or n)"}

begin

	string imroot, gcroot, imsec, imx
	string gcsect0, gcsect1, gcsect2, gcsect3, gcsect4
	file imin, imu
	bool vb, db, dt, sv

	vb = verbose
	db = debug
	dt = deltemp
	sv = saveold
	imin = image
	imx = ".fits"

#  If amplifier gain has already been done, return
	imgetp.default = "0" 
	imsec = imin // "[1]"
	imgetp (imsec, "GAINCORR")
	if (imgetp.value != "0") {
	    print ("HPROC: GAINCORR has already been run on", imin)
	    print ("GAINCORR: ", imgetp.value)
	    return
	    }

	if(vb) printf("GAINCORR: imarith on extensions of %s\n",imin)
	
# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imin
	    imin = imin // imx
	    }
   
# Set output file name root
	gcroot = imroot // "gc"

# Assign file names for extensions of output files
	gcsect0 = gcroot // "_0.fits"
	if (access (gcsect0))
	    delete (gcsect0)
	gcsect1 = gcroot // "_1.fits"
	if (access (gcsect1))
	    delete (gcsect1)
	gcsect2 = gcroot // "_2.fits"
	if (access (gcsect2))
	    delete (gcsect2)
	gcsect3 = gcroot // "_3.fits"
	if (access (gcsect3))
	    delete (gcsect3)
	gcsect4 = gcroot // "_4.fits"
	if (access (gcsect4))
	    delete (gcsect4)

# Multiply each amplifier image by appropriate gain correction
	imcopy (imin // "[0]", gcsect0, verbose=vb)
	imarith(imin // "[1]", '*', g1, gcsect1, verbose=db)
	imarith(imin // "[2]", '*', g2, gcsect2, verbose=db)
	imarith(imin // "[3]", '*', g3, gcsect3, verbose=db)
	imarith(imin // "[4]", '*', g4, gcsect4, verbose=db)

  # Add GAINCORR keyword to header
	hedit.show = no
	hedit (gcsect1, "GAINCORR", (g1), add=yes, verify=no)
	hedit (gcsect2, "GAINCORR", (g2), add=yes, verify=no)
	hedit (gcsect3, "GAINCORR", (g3), add=yes, verify=no)
	hedit (gcsect4, "GAINCORR", (g4), add=yes, verify=no)

#  Save original file in Ungain/ directory, if requested
	if (sv) {
	    imu = "Ungain/" // imin
	    if (access (imu))
		delete (imu)
	    rename (imin, imu)
	    }
	else {
	    delete (imin)
	    }

#  Rejoin result to same name as original file
	mscjoin (gcroot, output=imin, delete=dt, verbose=vb)
	if (vb)
	    print("GAINCORR: rejoined ",imin)

end

# Feb 22 2008	New task to gain correct one image, based on GAINCORRECT
# Feb 29 2008	Updated gains for Hectospec from Nelson Caldwell
