# HPREAMP1 - Run SPP version of Brian McCleod's preamplifier fix on raw image
# February 22, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hpreamp1 (image)

string	image=""	{prompt="Image to process"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}
bool	debug=no	{prompt="Print more process progress (yes or no)"}

begin

file	imroot, imin, templist, imout
string	imx, ims, imsec, history
string	pfsect0, pfsect1, pfsect2, pfsect3, pfsect4
bool	vb, dt, db, sv

	vb = verbose
	dt = deltemp
	db = debug
	sv = saveold
	imx = ".fits"
	imin = image

	if (imin == "") {
	    print ("HPREAMP1: Correct preamp problem for all four amps one amp at a time")
	    print ("Usage: hpreamp1 <input image with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (sv) {
	    if (!access ("Unpreamp"))
		mkdir ("Unpreamp")
	    }

# Do not process image without extensions
	imgetp.default = "-1"
	imsec = imin // "[0]"
	imgetp (imsec, "i_naxis")
	if (imgetp.value != "0") {
	    print ("HPREAMP1: ", imin, " is not a multi-extension image")
	    print ("NAXIS = ", imgetp.value)
	    return
	    }

# Do not process image twice
	imgetp.default = "0"
	imsec = imin // "[1]"
	imgetp (imsec, "PREAMPFX")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HPREAMP1: ", imin, " has already been processed")
		print ("PREAMPFX = ", imgetp.value)
		}
	    return
	    }

# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imageout
	    imin = imin // imx
	    }

# Fix preamp output using SPP version of Brian McLeod's C program preampfix
	if (db) {
	    print ("HPREAMP: Making preamp correction for ",imin)
	    print ("HPREAMP: image root is ",imroot)
	    }
	preamp1.verbose = vb
	preamp1.debug = db
	preamp1.error = ""
	preamp1.indir = ""
	pfsect0 = imroot // "_0.fits"
	imcopy (imin // "[0]", pfsect0, verbose=vb)
	pfsect1 = imroot // "_1.fits"
	imcopy (imin // "[1]", pfsect1, verbose=vb)
	preamp1 (pfsect1)
	if (preamp1.error != "")
	    print ("HPREAMP1 1: ",error)
	pfsect2 = imroot // "_2.fits"
	imcopy (imin // "[2]", pfsect2, verbose=vb)
	preamp1.error = ""
	preamp1 (pfsect2)
	if (preamp1.error != "")
	    print ("HPREAMP1 2: ",error)
	pfsect3 = imroot // "_3.fits"
	imcopy (imin // "[3]", pfsect3, verbose=vb)
	preamp1.error = ""
	preamp1 (pfsect3)
	if (preamp1.error != "")
	    print ("HPREAMP1 3: ",error)
	pfsect4 = imroot // "_4.fits"
	imcopy (imin // "[4]", pfsect4, verbose=vb)
	preamp1.error = ""
	preamp1 (pfsect4)
	if (preamp1.error != "")
	    print ("HPREAMP1 4: ",error)

# Save unchanged image
	if (sv) {
	    imout = "Raw/" // imin
	    if (!access (imout))
		rename (imin, imout)
	    else
		delete (imin)
	    }
	else
	    delete (imin)

# Reassemble the multi-extension FITS file
	if(vb)
	    print("HPREAMP1: rejoining ",imin)

# Reassemble the input file with the fixed extensions
	mscjoin (imroot, output = imin, delete=yes, verbose=vb)

end

# Feb 22 2008	New program to process a single image, based on HPREAMP
