# HPROC - Run image processing tasks one amp at a time
# February 29, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hproc (images)

string	images=""	{prompt="Images to process"}

bool	fixpreamp=yes	{prompt="Run preampfix to clean overscan (y or n)"}
string	biassec="[1080:1120,3:4610]"	{prompt="Bias section"}
bool	fixpix=yes	{prompt="Fix bad pixels (y or n)"}
bool	gain=yes	{prompt="Correct for variation in gain between preamps (y or n)"}
real	g1 = 1.0	{prompt="Gain correction for amplifier 1"}
real	g2 = 1.01	{prompt="Gain correction for amplifier 2"}
real	g3 = 1.032	{prompt="Gain correction for amplifier 3"}
real	g4 = 1.042	{prompt="Gain correction for amplifier 4"}
bool	merge=no	{prompt="Merge multi-extension images into one or two (y or n)"}
bool	perchip=no	{prompt="Merge amps to one output file per CCD (y or n)"}
bool	flatten=no	{prompt="Flat field correct spectra using HFLAT (y or n)"}
file	normfile=""	{prompt="Pixel to pixel normalization file from MAKEFLAT"}
bool	cosmic=no	{prompt="Remove cosmic rays from a list of images (yes or no)"}
real	threshold=20.0	{prompt="Threshold for cosmic ray difference"}
bool	recompute=no	{prompt="Recompute median if already there (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	saveold=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *proclist

begin

file	imin, templist, imu, nf
string	ims, imsec, temparg
bool	vb, fixpre, dt, sv, flat, rc
real	th
int	nfiles

	vb = verbose
	dt = deltemp
	sv = saveold
	nf = normfile
	th = threshold
	rc = recompute
	ims = images
	flat = flatten
	fixpre = fixpreamp

	if (ims == "") {
	    print ("HPROC: Run image processing tasks one amp at a time")
	    print ("Usage: hproc <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (sv) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	templist = mktemp ("tempproc") // ".list"
	files (ims, >templist)

# Process list of files
	nfiles = 0
	proclist = templist
        while (fscan (proclist, imin) != EOF) {

# Skip file if it cannot be read
	    smatch (imin, ".fits")
            if (!smatch.found) {
                imin = imin // ".fits"
                }
	    if (!access (imin)) {
		print ("HPROC: ", imin, " cannot be read")
		next
		}
	    nfiles = nfiles + 1

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("HPROC: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Save Raw file, if requested
	    if (sv) {
		imu = "Raw/" // imin
		if (access (imu))
		    delete (imu)
		copy (imin, imu)
		}

# Fix preamp output using Brian McLeod's task
	    if (fixpre)
		hpreamp1 (imin,saveold=no,deltemp=dt,verbose=vb)

#  Run CCDPROC only if it has not already been run
	    hproc1 (imin,biassec=biassec,fixpix=fixpix,saveold=sv,verbose=vb)

#  Correct for amplifier gain only if it has not already been done
	    if (gain)
		hgain1 (imin,g1=g1,g2=g2,g3=g3,g4=g4,saveold=sv,deltemp=dt,verbose=vb)

# Merge amplifier image extensions into a single image (or two if perchip=yes)
	    if (merge)
		hmerge1 (imin,perchip=perchip,saveold=sv,deltemp=dt,verbose=vb)

# Flatten image by dividing by flat field file made by MAKEFLAT
	    if (flat)
		hflat1 (imin,normfile=nf,saveold=sv,verbose=vb)
	    }

# Remove cosmic rays, if requested
# To use this option, the files in the list must all have the same configuration
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		hcosmic (temparg, threshold=th,recompute=rc,deltemp=dt,saveold=sv,verbose=vb)
	    }
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }

	if (dt)
	    delete (templist)
end

# May 20 2004	New program to process list of multiextension images
# May 21 2004	Add header line for preamp fix; move original file to Raw/
# Jun  8 2004	Change name of task from PROCAMPS to HPROC
# Jun  8 2004	Save preampfixed, unccdproc'ed files to Unproc/
# Jun 23 2004	Save unprocessed files only if saveold=yes
# Sep  2 2004	Add HPROC keyword and skip file if it exists already
# Oct 13 2004	Write HPROC keyword to each amplifier's image extension
# Oct 18 2004	Print HPROC keyword if it is present
# Dec 17 2004	Skip images where NAXIS in primary header is non-zero
# Dec 20 2004	Skip preampfix if fixpreamp parameter is no
#
# Jan 25 2005	Add fixpix parameter to set ccdproc.fixpix

# Nov  1 2007	Call hpreamp instead of C preampfix program

# Feb 20 2008	Drop use of file temp1; it is not needed
# Feb 20 2008	Check for CCDPROC and GAINCORR in headers
# Feb 20 2008	Save intermediate results in Unproc/ and Ungain/
# Feb 20 2008	Read from proclist instead of list
# Feb 22 2008	Use HPROC1 to run mscred.ccdproc
# Feb 22 2008	Use HGAIN1 instead of GAINCORRxtension operations
# Feb 22 2008	Use HMERGE1 to complete multi-extension operations
# Feb 22 2008	Use HFLAT1 to flatten image
# Feb 26 2008	Add HCOSMIC to remove cosmic rays from list of images
# Feb 29 2008	Make gain correction optional and include factors as parameters
