# T_REREF -- Read reference features, shift, and stretch

include "identify.h"

procedure t_reref

char	refin[SZ_FNAME]		# Input reference image
char	refout[SZ_FNAME]	# Output reference image

int	verbose
int	i, nfeatures
int	ap[2]
double	shift, stretch, refpix, dindef, line0, line1
int	ir
#int	clgeti()
bool	clgetb()
double	clgetd()
pointer	id		# ID pointer
int	strsearch()
errchk	id_dbread, id_map, id_dbsave

begin
	call printf ("REREF: Starting program.\n")
	call flush (STDOUT)

        dindef = INDEFD

#  Read reference image name
#	call strcpy ("0003.comp.fits", refin, SZ_FNAME)
	call clgstr ("refin", refin, SZ_FNAME)
	call printf ("REREF: Input reference spectrum %s.\n")
	    call pargstr (refin)
	call flush (STDOUT)

# Allocate the basic data structures.
	call id_init (id)
	call printf ("REREF: ID structure initialized.\n")
	call flush (STDOUT)
	call ic_open (ID_IC(id))
	call printf ("REREF: ID structure opened.\n")
	call flush (STDOUT)

# Open the image and return if there is an error.
	call strcpy (refin, ID_IMAGE(id), ID_LENSTRING)
	iferr (call id_map (id)) {
	    call error (0, "Cannot read reference image")
	    return
	    }
	iferr (call id_dbsave (id, ID_IMAGE(id))) {
	    call printf ("REREF: Cannot read reference image line IDs\n")
#	    call error (0, "Cannot read reference image line IDs")
#	    return
	    }
	call printf ("REREF: Image %s opened.\n")
	    call pargstr (refin)
	call flush (STDOUT)

# Get and save the reference database entry.
	call strcpy ("database", ID_DATABASE(id), ID_LENSTRING)
	call id_dbread (id, ID_IMAGE(id), ID_AP(id,1), NO, YES)
	call printf ("REREF: Image database read.\n")
	call flush (STDOUT)
	call id_saveap (id)
	call printf ("REREF: Image database saved.\n")
	call flush (STDOUT)

# Read shift, stretch, and reference pixel
	shift = clgetd ("shift")
	stretch = clgetd ("stretch")
	refpix = clgetd ("refpix")
	verbose = clgetb ("verbose")

	nfeatures = ID_NFEATURES(id)
	if (nfeatures == 0)
	    call error (0, "No features in reference")
	else if (verbose) {
	    call printf ("REREF: %d features in reference.\n")
		call pargi (nfeatures)
	    }

# For each reference feature a shift is added to bring the pixel
# position near that for the image being identified
	do i = 1, nfeatures {
	    line0 = PIX(id,i)
	    line1 = line0 + (shift + (line0 - refpix) * stretch)
	    if (verbose) {
		call printf ("%2d: %.3f => %.3f %.3f %.3f %.3f\n")
		    call pargi (i)
		    call pargd (PIX(id,i))
		    call pargd (line1)
		    call pargd (FIT(id,i))
		    call pargd (USER(id,i))
		    call pargd (WTS(id,i))
		}
		PIX(id,i) = line1
	    }

# Write revised pixel numbers to the output database
	call clgstr ("refout", refout, SZ_FNAME)
	call printf ("REREF: Output reference spectrum %s.\n")
	    call pargstr (refout)
	call flush (STDOUT)
	call strcpy (refout, ID_IMAGE(id), ID_LENSTRING)
	ap[1] = 1
	ap[2] = 0
	ir = strsearch (refout, ".fits")
	if (ir > 0)
	    refout[ir-5] = EOS
	else {
	    ir = strsearch (refout, ".imh")
	    if (ir > 0)
		refout[ir-4] = EOS
	    }
	call id_dbwrite (id, refout, ap, verbose)

end
